import tkinter as tk
from tkinter import filedialog
from PIL import Image, ImageTk
import numpy as np
from scipy.ndimage import label, find_objects, binary_dilation
import os
import glob

# Глобальные переменные
image = None
image_path = None
tk_image = None
history = []  # Stack to keep track of image history
scale = None
canvas = None
cluster_size_var = None
min_distance_var = None

def open_file():
    global image, image_path, history
    file_path = filedialog.askopenfilename(filetypes=[("BMP files", "*.bmp")])
    if file_path:
        image_path = file_path
        image = Image.open(file_path)
        history = [image.copy()]  # Initialize history with the original image
        display_image()

def save_file():
    global image, image_path
    if image and image_path:
        image.save(image_path, dpi=image.info.get('dpi', (600, 600)))

def display_image():
    global image, tk_image
    if image:
        zoom_percent = scale.get()
        new_size = (int(image.width * zoom_percent / 100), int(image.height * zoom_percent / 100))
        resized_image = image.resize(new_size, Image.Resampling.LANCZOS)
        tk_image = ImageTk.PhotoImage(resized_image)
        canvas.create_image(0, 0, anchor=tk.NW, image=tk_image)
        canvas.config(scrollregion=canvas.bbox(tk.ALL))

def zoom_image(scale_value):
    display_image()

def delete_clusters():
    global image, history
    if image:
        # Save the current image to history before making changes
        history.append(image.copy())

        # Convert indexed image to grayscale
        image = image.convert("L")  # 'L' mode is for grayscale
        image_array = np.array(image)
        
        # Create a mask for non-white pixels (white in grayscale is 255)
        non_white_mask = image_array != 255

        # Label connected components (clusters)
        labeled_array, num_features = label(non_white_mask)
        print(f"Found {num_features} clusters in the image.")  # Debug: number of clusters

        # Find objects and filter clusters by size
        cluster_slices = find_objects(labeled_array)
        max_cluster_size = cluster_size_var.get()
        min_distance = min_distance_var.get()

        to_delete = []

        for i, slice_obj in enumerate(cluster_slices, start=1):
            cluster = labeled_array[slice_obj] == i
            cluster_size = cluster.sum()

            if cluster_size <= max_cluster_size:
                x_start, x_end = slice_obj[1].start, slice_obj[1].stop
                y_start, y_end = slice_obj[0].start, slice_obj[0].stop

                # Check if the cluster is isolated
                padding_slice = (slice(max(0, y_start - min_distance), min(y_end + min_distance, non_white_mask.shape[0])),
                                 slice(max(0, x_start - min_distance), min(x_end + min_distance, non_white_mask.shape[1])))

                neighborhood = non_white_mask[padding_slice]
                neighborhood_cluster = labeled_array[padding_slice] == i

                # Increase the tolerance for finding clusters with white pixels
                dilated_cluster = binary_dilation(neighborhood_cluster, structure=np.ones((3, 3)))
                if np.any(dilated_cluster & neighborhood):
                    to_delete.append(i)
                    print(f"Cluster {i} marked for deletion. Size: {cluster_size}")  # Debug: cluster size and deletion

        # Delete the identified clusters by setting their pixels to white (255 in grayscale)
        to_delete_mask = np.isin(labeled_array, to_delete)
        image_array[to_delete_mask] = 255
        print(f"Deleting {len(to_delete)} clusters.")

        # Convert NumPy array back to image and update display
        image = Image.fromarray(image_array)
        display_image()

        # Confirm the deletion was applied
        if len(to_delete) > 0:
            print("Clusters deleted successfully.")
        else:
            print("No clusters deleted.")



def undo():
    global image, history
    if len(history) > 1:
        history.pop()  # Remove the last state
        image = history[-1]  # Restore the previous state
        display_image()
        print("Undo successful.")
    else:
        print("No action to undo.")

def close_file():
    global image, image_path, tk_image, history
    image = None
    image_path = None
    tk_image = None
    canvas.delete("all")
    history = []  # Clear history
    print("File closed.")

def process_folder():
    global image, image_path, history
    folder_path = filedialog.askdirectory()
    if folder_path:
        bmp_files = glob.glob(os.path.join(folder_path, "*.bmp"))
        for file_path in bmp_files:
            print(f"Processing file: {file_path}")
            image_path = file_path
            image = Image.open(file_path)
            history = [image.copy()]  # Initialize history with the original image
            delete_clusters()  # Process and delete clusters
            save_file()  # Save the processed image
        print("All files processed.")

# GUI Setup
root = tk.Tk()
root.title("BMP Image Editor")

canvas = tk.Canvas(root)
canvas.pack(expand=True, fill=tk.BOTH)

scale = tk.Scale(root, from_=10, to=400, orient=tk.HORIZONTAL, label="Zoom", command=zoom_image)
scale.set(30)  # Default zoom at 30%
scale.pack()

cluster_size_var = tk.IntVar(value=5)
cluster_size_entry = tk.Entry(root, textvariable=cluster_size_var)
cluster_size_entry.pack(side=tk.LEFT, padx=10, pady=10)

min_distance_var = tk.IntVar(value=5)
min_distance_entry = tk.Entry(root, textvariable=min_distance_var)
min_distance_entry.pack(side=tk.LEFT, padx=10, pady=10)

open_button = tk.Button(root, text="Open", command=open_file)
open_button.pack(side=tk.LEFT, padx=10, pady=10)

save_button = tk.Button(root, text="Save", command=save_file)
save_button.pack(side=tk.LEFT, padx=10, pady=10)

delete_clusters_button = tk.Button(root, text="Delete Clusters", command=delete_clusters)
delete_clusters_button.pack(side=tk.LEFT, padx=10, pady=10)

undo_button = tk.Button(root, text="Undo", command=undo)
undo_button.pack(side=tk.LEFT, padx=10, pady=10)

close_button = tk.Button(root, text="Close File", command=close_file)
close_button.pack(side=tk.LEFT, padx=10, pady=10)

process_folder_button = tk.Button(root, text="Process Folder", command=process_folder)
process_folder_button.pack(side=tk.LEFT, padx=10, pady=10)

root.mainloop()
